{
    "name": "{{ project_name }}",
    "startFile": "{{ project_name }}.asaql",
    "configurations": [
        {
            "filePath": "JobConfig.json",
            "subType": "JobConfig"
        }
    ]
}