import dotenv from 'dotenv';

dotenv.config();

export const config = {
  PORT: process.env.PORT || 3000,
  KAFKA_BROKERS: process.env.KAFKA_BROKERS || 'localhost:9092',
  KAFKA_TOPIC: process.env.KAFKA_TOPIC || 'noaa-events',
  CONSUMER_GROUP: process.env.CONSUMER_GROUP || 'default',
  KAFKA_USERNAME: process.env.KAFKA_USERNAME || '',
  KAFKA_PASSWORD: process.env.KAFKA_PASSWORD || '',
};
