{%- set schema = root -%}
{%- set namespace = class_name | namespace(project_name) -%}
{
    {%- if 'namespace' not in schema %}
        "namespace": "{{ namespace }}",
    {%- endif %}    
    {%- for attr, value in schema.items() %}
    "{{ attr }}": {{ value | tojson(3) }},
    {%- endfor %}
}