{%- set schema_name = class_name -%}
syntax="proto3";

option csharp_namespace = "{{ schema_name | namespace(project_name) }}";

{{ root | proto | regex_replace('syntax[\\s]*=[\\s]*["\']proto3["\'];[\\s]*\n', '') }}