{
  "name": "{{ project_name | lower | replace('.', '-') }}-eventgrid-producer",
  "version": "1.0.0",
  "type": "module",
  "description": "Azure Event Grid Producer for {{ project_name }}",
  "main": "dist/index.js",
  "types": "dist/index.d.ts",
  "scripts": {
    "build": "tsc",
    "test": "jest",
    "test:coverage": "jest --coverage",
    "clean": "rm -rf dist"
  },
  "dependencies": {
    "@azure/eventgrid": "^5.0.0",
    "@azure/identity": "^4.0.0",
    "cloudevents": "^8.0.2",
    "uuid": "^9.0.0"
  },
  "devDependencies": {
    "@jest/globals": "^29.7.0",
    "@types/jest": "^29.5.12",
    "@types/node": "^22.7.5",
    "@types/uuid": "^9.0.0",
    "jest": "^29.7.0",
    "nock": "^13.5.0",
    "ts-jest": "^29.2.5",
    "typescript": "^5.6.3"
  }
}
