import subprocess
import os

def main():
    packages = ["{{main_project_name}}", "{{data_project_name}}"]

    for package in packages:
        os.chdir(package)
        subprocess.run(["poetry", "build"], check=True)
        os.chdir("..")

if __name__ == "__main__":
    main()
