#!/bin/bash
set -e

echo "Installing {{ main_project_name }}..."

# Check if poetry is installed
if ! command -v poetry &> /dev/null; then
    echo "Poetry is not installed. Installing Poetry..."
    curl -sSL https://install.python-poetry.org | python3 -
fi

# Install dependencies
poetry install

echo "Installation complete!"
echo "Run 'poetry run pytest' to run tests"
