{
  "name": "{{ project_name | snake | replace('_', '-') }}",
  "version": "1.0.0",
  "description": "Kafka consumer for {{ project_name }}",
  "type": "module",
  "main": "dist/index.js",
  "types": "dist/index.d.ts",
  "scripts": {
    "build": "tsc",
    "watch": "tsc --watch",
    "test": "jest",
    "test:watch": "jest --watch",
    "test:coverage": "jest --coverage",
    "lint": "eslint 'src/**/*.ts' 'test/**/*.ts'",
    "lint:fix": "eslint 'src/**/*.ts' 'test/**/*.ts' --fix"
  },
  "keywords": ["kafka", "consumer", "cloudevents"],
  "author": "",
  "license": "MIT",
  "dependencies": {
    "kafkajs": "^2.2.4",
    "cloudevents": "^8.0.2"
  },
  "devDependencies": {
    "@jest/globals": "^29.7.0",
    "@types/jest": "^29.5.12",
    "@types/node": "^22.7.5",
    "@typescript-eslint/eslint-plugin": "^8.8.1",
    "@typescript-eslint/parser": "^8.8.1",
    "@stylistic/eslint-plugin": "^2.9.0",
    "eslint": "^9.12.0",
    "jest": "^29.7.0",
    "ts-jest": "^29.2.5",
    "ts-node": "^10.9.2",
    "typescript": "^5.6.3",
    "testcontainers": "^10.13.2",
    "@testcontainers/kafka": "^11.7.2"
  }
}
