<Project Sdk="Microsoft.NET.Sdk">
  <PropertyGroup>
    <TargetFramework>net9.0</TargetFramework>
    <ProjectName>{{project_name | pascal }}</ProjectName>
    <ImplicitUsings>enable</ImplicitUsings>
    <Nullable>enable</Nullable>
  </PropertyGroup>
  <ItemGroup>
    <ProjectReference Include="..\..\{{ data_project_name }}\src\{{data_project_name | pascal}}.csproj" />
  </ItemGroup>
  <ItemGroup>
    {{- dependency( 'cs','net90', 'CloudNative.CloudEvents') }}
    {{- dependency( 'cs','net90', 'CloudNative.CloudEvents.SystemTextJson') }}
    {{- dependency( 'cs','net90', 'CloudNative.CloudEvents.Protobuf') }}
    {{- dependency( 'cs','net90', 'CloudNative.CloudEvents.Avro') }}
    {{- dependency( 'cs','net90', 'Confluent.Kafka') }}
    {{- dependency( 'cs','net90', 'Microsoft.Extensions.Logging') }}
    {{- dependency( 'cs','net90', 'System.Memory.Data') }}
    {{- dependency( 'cs','net90', 'System.Text.Json') }}
  </ItemGroup>
</Project>