<Project Sdk="Microsoft.NET.Sdk">
  <PropertyGroup>
    <TargetFramework>net9.0</TargetFramework>
    <Nullable>enable</Nullable>
  </PropertyGroup>
  <ItemGroup>
    <ProjectReference Include="..\..\{{ data_project_name }}\src\{{data_project_name | pascal}}.csproj" />
  </ItemGroup>
  <ItemGroup>
      {{- dependency( 'cs','net90', 'Azure.Core') }}
      {{- dependency( 'cs','net90', 'Azure.Core.Amqp') }}
      {{- dependency( 'cs','net90', 'Azure.Messaging.EventGrid') }}
      {{- dependency( 'cs','net90', 'Microsoft.Extensions.Logging') }}
      {{- dependency( 'cs','net90', 'System.Text.Json') }}
  </ItemGroup>
  <ItemGroup>
    <None Update="host.json">
      <CopyToOutputDirectory>PreserveNewest</CopyToOutputDirectory>
    </None>
    <None Update="local.settings.json">
      <CopyToOutputDirectory>PreserveNewest</CopyToOutputDirectory>
      <CopyToPublishDirectory>Never</CopyToPublishDirectory>
    </None>
  </ItemGroup>

</Project>
