{%- macro dep(name) -%}{{- dependency( 'cs', 'net90', name) -}}{%- endmacro -%}
<Project Sdk="Microsoft.NET.Sdk">

  <PropertyGroup>
    <TargetFramework>net9.0</TargetFramework>
    <IsPackable>false</IsPackable>
    <Nullable>enable</Nullable>
  </PropertyGroup>

  <ItemGroup>
    {{- dep( 'Moq' )}}
    {{- dep( 'xunit' )}}
    {{- dep( 'xunit.runner.visualstudio' )}}
    {{- dep( 'Microsoft.NET.Test.Sdk' )}}
    {{- dep( 'Testcontainers' )}}
    {{- dep( 'Testcontainers.Azurite' )}}
    {{- dep( 'CloudNative.CloudEvents' )}}
    {{- dep( 'CloudNative.CloudEvents.SystemTextJson' )}}
    {{- dep( 'CloudNative.CloudEvents.Protobuf' )}}
    {{- dep( 'CloudNative.CloudEvents.Avro' )}}
    {{- dep( 'Azure.Messaging.ServiceBus' )}}
    {{- dep( 'Microsoft.Extensions.Logging' )}}
    {{- dep( 'Microsoft.Extensions.Logging.Console' )}}
    {{- dep( 'Microsoft.Extensions.Logging.Debug' )}}
    {{- dep( 'System.Memory.Data' )}}
  </ItemGroup>

  <ItemGroup>
    <ProjectReference Include="..\src\{{ main_project_name | pascal }}.csproj" />
    <ProjectReference Include="..\..\{{ data_project_name }}\test\{{data_project_name | pascal}}.Test.csproj" />
  </ItemGroup>

</Project>
