{%- import "util.jinja.include" as util -%}
{%- set messagegroups = root.messagegroups %}
// This code was generated by the xRegistry tool.
// Changes to this file may cause incorrect behavior and will be lost if the code is regenerated.

export { KafkaProcessor } from './tools';
{% for messagegroupid, messagegroup in messagegroups.items() -%}
{%- set messagegroupname = messagegroupid | pascal -%}
{%- set class_name = ( messagegroupname | strip_namespace ) + "EventDispatcher" %}
export { {{ class_name }} } from './dispatcher';
{%- endfor %}
