{%- import "util.jinja.include" as util -%}
module {{ project_name | snake }}

go 1.21

require (
	github.com/cloudevents/sdk-go/v2 v2.15.2
	github.com/segmentio/kafka-go v0.4.47
	github.com/google/uuid v1.6.0
	github.com/username/{{ data_project_name | snake }} v0.0.0
	github.com/testcontainers/testcontainers-go v0.27.0
	github.com/stretchr/testify v1.8.4
)

replace github.com/username/{{ data_project_name | snake }} => ../{{ data_project_name }}
